from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from .models import Asset, AssetCategory
from account.models import Account
from django.db import IntegrityError

from django.shortcuts import render, get_object_or_404
from .models import Asset, AssetIssueHistory

def asset_details(request, asset_id): 
    asset = get_object_or_404(Asset, id=asset_id) 
    issue_history = AssetIssueHistory.objects.filter(asset=asset).order_by('-date_updated')

    context = {
        'asset': asset,
        'issue_history': issue_history,
    }
    return render(request, 'inventory/asset/asset_details.html', context)

# AssetCategory CRUD
@login_required
def asset_categories(request):
    if request.user.is_admin_user or request.user.is_super_user:
        categories = AssetCategory.objects.all()
        return render(request, "inventory/asset/categories.html", {'categories': categories})
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def add_asset_category(request):
    print("Test")
    if request.user.is_admin_user or request.user.is_super_user:
        if request.method == "POST":
            name = request.POST.get('name')
            AssetCategory.objects.create(name=name)
            messages.success(request, "Category added successfully.")
            return redirect('inventory:add_asset')
        return redirect('inventory:add_asset')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")

@login_required
def edit_asset_category(request, category_id):
    category = get_object_or_404(AssetCategory, id=category_id)
    if request.user.is_admin_user or request.user.is_super_user:
        if request.method == "POST":
            name = request.POST.get('name')
            category.name = name
            category.save()
            messages.success(request, "Category updated successfully.")
            return redirect('inventory:asset_categories')
        return render(request, "inventory/category/edit_category.html", {'category': category})
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")

@login_required
def delete_asset_category(request, category_id):
    category = get_object_or_404(AssetCategory, id=category_id)
    if request.user.is_admin_user or request.user.is_super_user:
        category.delete()
        messages.success(request, "Category deleted successfully.")
        return redirect('inventory:asset_categories')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")

# Asset CRUD
from datetime import timedelta
from django.utils import timezone

from django.contrib import messages
from django.shortcuts import render, get_object_or_404
from django.http import HttpResponseForbidden
from datetime import timedelta
from .models import Asset, AssetIssueHistory

from django.contrib import messages
from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponseForbidden
from django.db import IntegrityError
from .models import Asset, AssetIssueHistory, Account
from datetime import timedelta

@login_required
def assets(request):
    accounts = Account.objects.all()
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        # Handle POST request to update asset status
        if request.method == "POST":
            asset_id = request.POST.get('asset_id')
            status = request.POST.get('status')
            user_account_id = request.POST.get('user_account')  # Getting selected user ID
            asset = get_object_or_404(Asset, id=asset_id)
            
            # Validating and updating the status of the asset
            if status not in dict(Asset.STATUS_CHOICES):
                messages.error(request, "Invalid status value.")
            else:
                asset.status = status
                if status == 'Returned':
                    return_date = request.POST.get('return_date')
                    
                    if not return_date:
                        # If return date is not provided, set it to one month after the issue date
                        return_date = asset.date_issued + timedelta(days=30)
                    
                    asset.return_date = return_date
                    asset.return_comments = request.POST.get('return_comments', '')
                
                # Save the asset after updating
                asset.save()

                # Check if a user is selected
                if user_account_id:
                    user_account = get_object_or_404(Account, id=user_account_id)
                    
                    # Check if the asset is already associated with this user in the history
                    existing_history = AssetIssueHistory.objects.filter(asset=asset, user_account=user_account).first()
                    
                    if existing_history:
                        # If an entry exists, update the history record
                        existing_history.user_account = user_account
                        existing_history.status = status
                        existing_history.return_comments = asset.return_comments
                        existing_history.save()
                    else:
                        # If no entry exists, create a new history record
                        AssetIssueHistory.objects.create(
                            asset=asset,
                            user_account=user_account,
                            status=status,
                            return_comments=asset.return_comments
                        )

                messages.success(request, "Asset status updated successfully.")

        # Retrieve all assets to display
        assets = Asset.objects.all()
        return render(request, "inventory/asset/assets.html", {'assets': assets, 'accounts':accounts})
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

from django.contrib import messages
from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponseForbidden
from django.db import IntegrityError
from .models import Asset, AssetCategory, Account, AssetIssueHistory
from datetime import datetime

@login_required
def add_asset(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        if request.method == "POST":
            try:
                asset_name = request.POST.get('asset_name')
                category_id = request.POST.get('category')
                serial_number = request.POST.get('serial_number')
                value = request.POST.get('value')
                description = request.POST.get('description')
                image = request.FILES.get('image')
                account_id = request.POST.get('account')
                date_issued = request.POST.get('date_issued') or datetime.now().date()
 
                # Retrieve the category and account objects
                category = get_object_or_404(AssetCategory, id=category_id) if category_id else None
                account = get_object_or_404(Account, id=account_id) if account_id else None

                # Create the asset record
                asset = Asset.objects.create(
                    asset_name=asset_name,
                    category=category,
                    serial_number=serial_number,
                    value=value,
                    description=description,
                    image=image,
                    account=account,
                    date_issued=date_issued,
                )

                # If a user is assigned to the asset, create an entry in the asset issue history
                if account:
                    AssetIssueHistory.objects.create(
                        asset=asset,
                        user_account=account,
                        status="Available",  # Assuming a newly added asset is "Available"
                        return_comments="",  # You can add any comments if needed
                    )

                messages.success(request, "Asset added successfully.")
                return redirect('inventory:add_asset')

            except IntegrityError as e:
                messages.error(request, f"Error: {str(e)}")
            except Exception as e:
                messages.error(request, f"An error occurred: {str(e)}")

        # Retrieve all categories and accounts to display in the form
        categories = AssetCategory.objects.all()
        accounts = Account.objects.all()

        return render(request, "inventory/asset/add_asset.html", {'categories': categories, 'accounts': accounts})
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")

@login_required
def edit_asset(request, asset_id):
    asset = get_object_or_404(Asset, id=asset_id)
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        if request.method == "POST":
            try:
                asset_name = request.POST.get('asset_name')
                category_id = request.POST.get('category')
                serial_number = request.POST.get('serial_number')
                value = request.POST.get('value')
                description = request.POST.get('description')
                image = request.FILES.get('image')
                account_id = request.POST.get('account')
                date_issued = request.POST.get('date_issued')

                asset.asset_name = asset_name
                asset.category = get_object_or_404(AssetCategory, id=category_id) if category_id else None
                asset.serial_number = serial_number
                asset.value = value
                asset.description = description
                asset.image = image if image else asset.image
                asset.account = get_object_or_404(Account, id=account_id) if account_id else None
                asset.date_issued = date_issued
                asset.save()

                messages.success(request, "Asset updated successfully.")
                return redirect('inventory:assets')
            except Exception as e:
                messages.error(request, f"An error occurred: {str(e)}")

        categories = AssetCategory.objects.all()
        accounts = Account.objects.all()
        return render(request, "inventory/asset/add_asset.html", {'asset': asset, 'categories': categories, 'accounts': accounts})
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")

@login_required
def delete_asset(request, asset_id):
    asset = get_object_or_404(Asset, id=asset_id)
    if request.user.is_admin_user or request.user.is_super_user:
        asset.delete()
        messages.success(request, "Asset deleted successfully.")
        return redirect('inventory:assets')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")
    

@login_required
def update_asset_status(request, asset_id):
    asset = get_object_or_404(Asset, id=asset_id)
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        if request.method == "POST":
            try:
                status = request.POST.get('status')
                return_comments = request.POST.get('return_comments', '')

                if status not in dict(Asset.STATUS_CHOICES):
                    messages.error(request, "Invalid status value.")
                else:
                    asset.status = status
                    if status == 'Returned':
                        asset.return_date = request.POST.get('return_date')  # Ensure return_date is passed in the form
                        asset.return_comments = return_comments
                    asset.save()

                    messages.success(request, "Asset status updated successfully.")
                    return redirect('inventory:assets')
            except Exception as e:
                messages.error(request, f"An error occurred: {str(e)}")

        return redirect('inventory:assets')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")

